(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react-mate'), require('lodash')) :
	typeof define === 'function' && define.amd ? define(['exports', 'react-mate', 'lodash'], factory) :
	(factory((global['react-mate-action-intercepting-integration'] = {}),global['react-mate'],global.lodash));
}(this, (function (exports,reactMate,_) { 'use strict';

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var ActionInterceptingPlugin = function () {
    function ActionInterceptingPlugin() {
        classCallCheck(this, ActionInterceptingPlugin);

        this.interceptors = [];
    }

    createClass(ActionInterceptingPlugin, [{
        key: "intercepting",
        value: function intercepting(filter, stringOrHandler) {
            this.interceptors.push(createActionInterceptor(filter, stringOrHandler));
            return this;
        }
    }, {
        key: "build",
        value: function build() {
            var _this = this;

            var actionInterceptingMiddleware = function actionInterceptingMiddleware(store) {
                return function (next) {
                    return function (action) {
                        var functionsToExecuteAfterwards = [];
                        var methods = {
                            block: function block() {
                                return true;
                            },
                            propagate: next,
                            dispatchAfterwards: function dispatchAfterwards(action) {
                                functionsToExecuteAfterwards.push(function () {
                                    return store.dispatch(action);
                                });
                            },
                            executeAfterwards: function executeAfterwards(func) {
                                functionsToExecuteAfterwards.push(func);
                            }
                        };

                        var atLeastOneInterceptorTriggered = false;
                        var _iteratorNormalCompletion = true;
                        var _didIteratorError = false;
                        var _iteratorError = undefined;

                        try {
                            for (var _iterator = _this.interceptors[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                                var _ref = _step.value;
                                var filter = _ref.filter,
                                    handler = _ref.handler;

                                if (filter(action, store)) {
                                    atLeastOneInterceptorTriggered = true;
                                    handler(action, store, methods);
                                }
                            }
                        } catch (err) {
                            _didIteratorError = true;
                            _iteratorError = err;
                        } finally {
                            try {
                                if (!_iteratorNormalCompletion && _iterator.return) {
                                    _iterator.return();
                                }
                            } finally {
                                if (_didIteratorError) {
                                    throw _iteratorError;
                                }
                            }
                        }

                        if (functionsToExecuteAfterwards.length > 0) {
                            var _iteratorNormalCompletion2 = true;
                            var _didIteratorError2 = false;
                            var _iteratorError2 = undefined;

                            try {
                                for (var _iterator2 = functionsToExecuteAfterwards[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                                    var functionToExecute = _step2.value;

                                    functionToExecute();
                                }
                            } catch (err) {
                                _didIteratorError2 = true;
                                _iteratorError2 = err;
                            } finally {
                                try {
                                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                                        _iterator2.return();
                                    }
                                } finally {
                                    if (_didIteratorError2) {
                                        throw _iteratorError2;
                                    }
                                }
                            }
                        }
                        if (!atLeastOneInterceptorTriggered) {
                            next(action);
                        }
                    };
                };
            };

            return reactMate.aPluginNamed('ActionInterceptingPlugin').withMiddleware(actionInterceptingMiddleware);
        }
    }]);
    return ActionInterceptingPlugin;
}();

function createActionInterceptor(filter, stringOrHandler) {
    if (!filter) {
        throw new Error("Missing filter function for ActionInterceptingPlugin.");
    }
    if (!_.isFunction(filter)) {
        throw new Error("Got an invalid filter function for ActionInterceptingPlugin.");
    }
    if (!stringOrHandler) {
        throw new Error("Missing handler function for ActionInterceptingPlugin.");
    }
    var handlerFunction = void 0;
    if (_.isFunction(stringOrHandler)) {
        handlerFunction = stringOrHandler;
    } else if (_.isString(stringOrHandler)) {
        handlerFunction = function handlerFunction(action) {
            return action.type === stringOrHandler;
        };
    } else {
        throw new Error("Got an invalid handler function for ActionInterceptingPlugin. Expected a function or a string");
    }
    return {
        filter: filter,
        handler: handlerFunction
    };
}

var anActionInterceptorPlugin = function anActionInterceptorPlugin() {
    return new ActionInterceptingPlugin();
};

exports.anActionInterceptorPlugin = anActionInterceptorPlugin;

Object.defineProperty(exports, '__esModule', { value: true });

})));
